package cz.cvut.fel.agents.pdv.evaluation;

import cz.cvut.fel.agents.pdv.evaluation.leader.ClientCoordinatorForLeaderSelection;
import cz.cvut.fel.agents.pdv.evaluation.leader.ScenarioForLeaderSelection;
import cz.cvut.fel.agents.pdv.evaluation.log.ClientCoordinatorForLogReplication;
import cz.cvut.fel.agents.pdv.evaluation.log.ScenarioForLogReplication;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Vasi implementaci budeme testovat ve 3 scenarich (s ruznou konfiguraci systemu). Nasim cilem je,
 * abyste zvladli implementovat jednoduchou distribuovanou key/value databazi s garancemi podle RAFT
 * (3. scenar). Scenare na sebe navazuji a dovedou Vas az k implementaci databaze.
 *
 * Spusteni testovacich scenaru. Zatim jsou pritomny pouze 2 (bude i 3) s ukazkami zakladni
 * konfigurace (pozdeji blize upresnime, jake konfiguraceme budeme v jednotlivych scenarich
 * testovat).
 *
 * Zacnete tim, ze se podivate na student.ClusterProcess, kde najdete dalsi navod a podivate se na
 * implementace scenaru a co delaji jednotlive parametry. Parametry jednotlivych scenaru upresnime
 * pozdeji.
 */
public class RaftRun {

  public static void main(String[] args) {

    printLineDelimiter(20);

    // Scenar Volba lidra
    ScenarioForLeaderSelection scenarioForLeaderSelection = ScenarioForLeaderSelection.builder()
        .requestPeriod(1)
        .countOfFailures(1)
        .lengthOfFailure(20)
        .spanBetweenFailures(50)
        .expectedNumberOfProcessesInPartition(3)
        .reconnectProcess(false)
        .build();
    RaftSimulation<ClientCoordinatorForLeaderSelection> simulationLeader = new RaftSimulation<>(
        new RaftDSConfiguration<>(scenarioForLeaderSelection, "Volba lidra"));
    simulationLeader.run();

    printLineDelimiter(20);

    // Scenar Replikace logu napric clusterem
    ScenarioForLogReplication scenarioForLogReplication = ScenarioForLogReplication.builder()
        .countOfFailures(1)
        .lengthOfFailure(20)
        .spanBetweenFailures(50)
        .expectedNumberOfProcessesInPartition(3)
        .stepsOfSimulation(200)
        .stopRequestBeforeEndOfSimulation(20)
        .reconnectProcess(true)
        .retryInterval(1)
        .build();
    RaftSimulation<ClientCoordinatorForLogReplication> simulationLog = new RaftSimulation<>(
        new RaftDSConfiguration<>(scenarioForLogReplication, "Replikace logu napric clusterem"));
    simulationLog.run();

    printLineDelimiter(20);

  }

  /**
   * Vytiskne oddelovac radku pro lepsi prehlednost mezi senari
   */
  private static void printLineDelimiter(int count) {
    String del = IntStream.range(0, count)
        .boxed()
        .map(i -> "-")
        .collect(Collectors.joining(""));
    System.out.println(del);
  }

}
